@@ -7,7 +7,7 @@
 #' Fully Conditional Specification multiple imputation approach proposed by
 #' Bartlett \emph{et al} 2015 (see references).
 #'
-#' Currently imputation is supported for linear regression (\code{"lm"}),
+#' Imputation is supported for linear regression (\code{"lm"}),
 #' logistic regression (\code{"logistic"}), Poisson regression
 #' (\code{"poisson"}), Weibull (\code{"weibull"}) and Cox regression
 #' for time to event data (\code{"coxph"}),
@@ -41,9 +41,9 @@
 #' @param originaldata The original data frame with missing values.
 #' @param smtype A string specifying the type of substantive model. Possible
 #' values are \code{"lm"}, \code{"logistic"}, \code{"poisson"}, \code{"weibull"},
-#' \code{"coxph"} and \code{"compet"}.
-#' @param smformula The formula of the substantive model. For \code{"weibull"} and \code{"coxph"} substantive
-#' models the left hand side should be of the form \code{"Surv(t,d)"}. For \code{"compet"}
+#' \code{"coxph"}, \code{"compet"}.
+#' @param smformula The formula of the substantive model. For \code{"weibull"} and \code{"coxph"}
+#' substantive models the left hand side should be of the form \code{"Surv(t,d)"}. For \code{"compet"}
 #' substantive models, a list should be passed consisting of the Cox models
 #' for each cause of failure (see example).
 #' @param method A required vector of strings specifying for each variable either
@@ -91,14 +91,14 @@
 #' \code{smCoefIter} a three dimension matrix containing the substantive model parameter
 #' values. The matrix is indexed by [imputation,parameter number,iteration]
 #'
-#' @author Jonathan Bartlett \email{j.w.bartlett@@bath.ac.uk} \url{http://www.missingdata.org.uk}
-#' \url{http://thestatsgeek.com}
+#' @author Jonathan Bartlett \email{j.w.bartlett@@bath.ac.uk} \url{https://thestatsgeek.com}
+#' \url{http://www.missingdata.org.uk}
 #'
 #' @example data-raw/examples.r
 #'
 #' @references Bartlett JW, Seaman SR, White IR, Carpenter JR. Multiple imputation of covariates
 #' by fully conditional specification: accommodating the substantive model. Statistical Methods
-#' in Medical Research 2015; 24(4): 462-487. \url{https://doi.org/10.1177/0962280214521348}
+#' in Medical Research 2015; 24(4): 462-487. \doi{10.1177/0962280214521348}
 
 #' @import stats
 
@@ -164,9 +164,43 @@ smcfcs.nestedcc <- function(originaldata,smformula,set,event,nrisk,method,predic
               errorProneMatrix=errorProneMatrix)
 }
 
+#' Substantive model compatible fully conditional specification imputation of covariates for
+#' discrete time survival analysis
+#'
+#' Multiply imputes missing covariate values using substantive model compatible
+#' fully conditional specification for discrete time survival analysis.
+#'
+#' For this substantive model type, like for the other substantive model types, \code{smcfcs} expects the \code{originaldata} to have
+#' one row per subject. Variables indicating the discrete time of failure/censoring
+#' and the event indicator should be passed in \code{smformula}, as described.
+#'
+#' The default is to model the effect of time as a factor. This will not work in datasets where
+#' there is not at least one observed event in each time period. In such cases you must specify
+#' a simpler parametric model for the effect of time. At the moment you can specify either a linear or quadratic
+#' effect of time (on the log odds scale).
+#'
+#' @author Jonathan Bartlett \email{j.w.bartlett@@bath.ac.uk}
+#'
+#' @param originaldata The data in wide form (i.e. one row per subject)
+#' @param smformula A formula of the form "Surv(t,d)~x1+x2+x3", where t is the discrete time variable, d is the binary event
+#' indicator, and the covariates should not include time. The time variable should be
+#' an integer coded numeric variable taking values from 1 up to the final time period.
+#' @param timeEffects Specifies how the effect of time is modelled. \code{timeEffects="factor"} (the default) models time as a
+#' factor variable. \code{timeEffects="linear"} and \code{timeEffects="quad"} specify that time be modelled as a continuous
+#' linear or quadratic effect on the log odds scale respectively.
+#'
+#' @inheritParams smcfcs
+#' @example data-raw/dtsam_example.r
+#' @export
+smcfcs.dtsam <- function(originaldata,smformula,timeEffects="factor",method,predictorMatrix=NULL,m=5,numit=10,rjlimit=1000,noisy=FALSE,errorProneMatrix=NULL) {
+  smcfcs.core(originaldata,smtype="dtsam",smformula,method,predictorMatrix,m,numit,rjlimit,noisy,timeEffects=timeEffects,
+              errorProneMatrix=errorProneMatrix)
+}
+
 #this is the core of the smcfcs function, called by wrapper functions for certain different substantive models
 smcfcs.core <- function(originaldata,smtype,smformula,method,predictorMatrix=NULL,m=5,numit=10,rjlimit=1000,noisy=FALSE,errorProneMatrix=NULL,
                         ...) {
+
   #get extra arguments passed in ...
   extraArgs <- list(...)
 
@@ -178,7 +212,8 @@ smcfcs.core <- function(originaldata,smtype,smformula,method,predictorMatrix=NUL
   #create matrix of response indicators
   r <- 1*(is.na(originaldata)==0)
 
-  if ((smtype %in% c("lm", "logistic", "poisson", "coxph", "compet", "casecohort","nestedcc", "weibull"))==FALSE)
+  if ((smtype %in% c("lm", "logistic", "poisson", "coxph", "compet", "casecohort","nestedcc",
+                     "weibull","dtsam"))==FALSE)
     stop(paste("Substantive model type ",smtype," not recognised.",sep=""))
 
   #find column numbers of partially observed, fully observed variables, and outcome
@@ -199,6 +234,32 @@ smcfcs.core <- function(originaldata,smtype,smformula,method,predictorMatrix=NUL
     dCol <- (1:dim(originaldata)[2])[colnames(originaldata) %in% toString(as.formula(smformula)[[2]][[3]])]
     outcomeCol <- c(timeCol, dCol)
     d <- originaldata[,dCol]
+  } else if (smtype=="dtsam") {
+    timeCol <- (1:dim(originaldata)[2])[colnames(originaldata) %in% toString(as.formula(smformula)[[2]][[2]])]
+    dCol <- (1:dim(originaldata)[2])[colnames(originaldata) %in% toString(as.formula(smformula)[[2]][[3]])]
+    outcomeCol <- c(timeCol, dCol)
+    d <- originaldata[,dCol]
+    #determine cut points
+    cutPoints <- 1:max(originaldata[,timeCol])
+    nTimePoints <- length(cutPoints)
+
+    #check all time points are integers
+    if (!all(unique(originaldata[,timeCol]) == floor(unique(originaldata[,timeCol])))) {
+      stop("Your time variable must only take positive integer values.")
+    }
+    #check all time points are positive
+    if (any(unique(originaldata[,timeCol]) <= 0)) {
+      stop("Your time variable must only take positive integer values.")
+    }
+
+    #if factor time effects, check there are some events at each integer value
+    #so that model can fit
+    if (extraArgs$timeEffects=="factor") {
+      if (!identical(sort(unique(originaldata[,timeCol][originaldata[,dCol]==1])), as.numeric(cutPoints))) {
+        stop("You cannot fit a dtsam model with factor time effects since there are some periods with no events. See documentation for timeEffects argument for parametric alternatives")
+      }
+    }
+
   } else if (smtype=="compet") {
 
     timeCol <- (1:dim(originaldata)[2])[colnames(originaldata) %in% toString(as.formula(smformula[[1]])[[2]][[2]])]
@@ -597,6 +658,28 @@ smcfcs.core <- function(originaldata,smtype,smformula,method,predictorMatrix=NUL
             print(summary(ymod))
           }
         }
+        else if (smtype=="dtsam") {
+          #split data to long form
+          longData <- survival::survSplit(as.formula(smformula), data=imputations[[imp]], cut=cutPoints)
+          #fit logistic model
+          if (extraArgs$timeEffects=="factor") {
+            dtsamFormula <- paste(colnames(imputations[[imp]])[dCol],"~-1+factor(tstart)+",
+                                strsplit(smformula, "~")[[1]][2],sep="")
+          } else if (extraArgs$timeEffects=="linear") {
+            #linear time effect
+            dtsamFormula <- paste(colnames(imputations[[imp]])[dCol],"~tstart+",
+                                  strsplit(smformula, "~")[[1]][2],sep="")
+          } else {
+            #quadratic effect of time
+            dtsamFormula <- paste(colnames(imputations[[imp]])[dCol],"~tstart+I(tstart^2)+",
+                                  strsplit(smformula, "~")[[1]][2],sep="")
+          }
+          ymod <- glm(as.formula(dtsamFormula), family="binomial", data=longData)
+          outcomeModBeta = modPostDraw(ymod)
+          if (noisy==TRUE) {
+            print(summary(ymod))
+          }
+        }
         else if (smtype=="poisson") {
           ymod <- glm(as.formula(smformula),family="poisson",imputations[[imp]])
           outcomeModBeta = modPostDraw(ymod)
@@ -739,6 +822,10 @@ smcfcs.core <- function(originaldata,smtype,smformula,method,predictorMatrix=NUL
               prob <- expit(outmodxb[imputationNeeded])
               outcomeDens <- prob*imputations[[imp]][imputationNeeded,outcomeCol] + (1-prob)*(1-imputations[[imp]][imputationNeeded,outcomeCol])
             }
+            else if (smtype=="dtsam") {
+              outcomeDens <- dtsamOutcomeDens(imputations[[imp]],extraArgs$timeEffects,outcomeModBeta,nTimePoints
+                                              ,smformula,timeCol,dCol)[imputationNeeded]
+            }
             else if (smtype=="poisson") {
               outmodxb <-  model.matrix(as.formula(smformula),imputations[[imp]]) %*% outcomeModBeta
               outcomeDens <- dpois(imputations[[imp]][imputationNeeded,outcomeCol], exp(outmodxb[imputationNeeded]))
@@ -819,6 +906,11 @@ smcfcs.core <- function(originaldata,smtype,smformula,method,predictorMatrix=NUL
               prob = prob*imputations[[imp]][imputationNeeded,outcomeCol] + (1-prob)*(1-imputations[[imp]][imputationNeeded,outcomeCol])
               reject = 1*(uDraw>prob)
             }
+            else if (smtype=="dtsam") {
+              prob <-  dtsamOutcomeDens(imputations[[imp]],extraArgs$timeEffects,outcomeModBeta,nTimePoints
+                                        ,smformula,timeCol,dCol)[imputationNeeded]
+              reject = 1*(uDraw>prob)
+            }
             else if (smtype=="poisson") {
               outmodxb <-  model.matrix(as.formula(smformula),imputations[[imp]]) %*% outcomeModBeta
               prob = dpois(imputations[[imp]][imputationNeeded,outcomeCol], exp(outmodxb[imputationNeeded]))
@@ -893,6 +985,11 @@ smcfcs.core <- function(originaldata,smtype,smformula,method,predictorMatrix=NUL
               prob = prob*tempData[,outcomeCol] + (1-prob)*(1-tempData[,outcomeCol])
               reject = 1*(uDraw>prob)
             }
+            else if (smtype=="dtsam") {
+              prob <-  dtsamOutcomeDens(tempData,extraArgs$timeEffects,outcomeModBeta,nTimePoints
+                                        ,smformula,timeCol,dCol)
+              reject = 1*(uDraw>prob)
+            }
             else if (smtype=="poisson") {
               outmodxb <-  model.matrix(as.formula(smformula),tempData) %*% outcomeModBeta
               prob = dpois(tempData[,outcomeCol], exp(outmodxb))
@@ -1021,3 +1118,47 @@ modPostDraw <- function(modobj) {
   beta + MASS::mvrnorm(1, mu=rep(0,ncol(varcov)), Sigma=varcov)
 }
 
+#a helper function which calculates the substantive model probability value
+#for each individual for the discrete time survival (logistic) substantive model
+dtsamOutcomeDens <- function(inputData,timeEffects,outcomeModBeta,nTimePoints,smformula,timeCol,dCol) {
+  inputDataN <- dim(inputData)[1]
+
+  if (timeEffects=="factor") {
+    #first add in time effects on log odds scale
+    outmodxb <- matrix(outcomeModBeta[1:nTimePoints], nrow=inputDataN, ncol=nTimePoints,byrow=TRUE)
+    #calculate covariate effects
+    covXbEffects <-  model.matrix(as.formula(paste("~-1+",strsplit(smformula, "~")[[1]][2],sep="")),
+                                  inputData) %*% utils::tail(outcomeModBeta,length(outcomeModBeta)-nTimePoints)
+  } else if (timeEffects=="linear") {
+    #linear time
+    #first add in time effects on log odds scale
+    outmodxb <- outcomeModBeta[1] +
+      matrix(outcomeModBeta[2]*((1:nTimePoints)-1), nrow=inputDataN, ncol=nTimePoints,byrow=TRUE)
+    #calculate covariate effects
+    covXbEffects <-  model.matrix(as.formula(paste("~-1+",strsplit(smformula, "~")[[1]][2],sep="")),
+                                  inputData) %*% utils::tail(outcomeModBeta,length(outcomeModBeta)-2)
+  } else {
+    #quadratic time
+    #first add in time effects on log odds scale
+    outmodxb <- outcomeModBeta[1] +
+      matrix(outcomeModBeta[2]*((1:nTimePoints)-1)+outcomeModBeta[3]*(((1:nTimePoints)-1)^2), nrow=inputDataN, ncol=nTimePoints,byrow=TRUE)
+    #calculate covariate effects
+    covXbEffects <-  model.matrix(as.formula(paste("~-1+",strsplit(smformula, "~")[[1]][2],sep="")),
+                                  inputData) %*% utils::tail(outcomeModBeta,length(outcomeModBeta)-3)
+  }
+
+  #add in covariate effects
+  outmodxb <- outmodxb + matrix(covXbEffects, nrow=inputDataN, ncol=nTimePoints)
+  #prob is matrix of conditional probabilities/hazard of event in each period
+  prob <- expit(outmodxb)
+  logSurvProb <- log(1-prob)
+  logSurvProbCumSum <- cbind(rep(0,inputDataN),t(apply(logSurvProb, 1, cumsum)))
+
+  #create vector of last time point each person survived to, +1
+  lastSurvPlusOne <- 1 + inputData[,timeCol] - inputData[,dCol]
+  #create vector of log of probability of survival to time when each person actually survived to
+  logSurvProbIndividual <- logSurvProbCumSum[cbind(1:inputDataN,lastSurvPlusOne)]
+  #return vector of outcome density values
+  exp(logSurvProbIndividual + inputData[,dCol]*log(prob[cbind(1:inputDataN, inputData[,timeCol])]))
+
+}
